--[[
  Log file handling. 
 
  See logtest.lua for usage examples.

    
  Author: fudgey 2008/11/03
  Licence: GPL v3 or higher
--]]

log={}

--[[
    Open log CHDK/LOGS/name for append or overwrite if io is available.
    If name==nil, use lua_NNN.txt where NNN is the smallest free 3-digit number.
    If aw=nil or "a", open for append. If aw="w", open for write.
    
    Returns a table (use names it freely, let's call it "mylog" here) which
    contains:
        mylog:print()   -- function to print a string and a newline to this log 
                           file WARNING! Do not pass nil as an argument! Use 
                           tostring() to prevent it whenever necessary.
        mylog:close()   -- function to close this log file
        mylog.handle    -- file handle to the log, in case you want to do 
                           something more advanced to the file. Value is nil
                           when the library is using print_screen for logging.
        mylog.name      -- name of this log file (without path).
        mylog.path      -- path of this log file (without filename).
        mylog.logtype   -- value is string "io_write" if using io functions,
                           "print_screen" if io is not available and using 
                           print_screen instead.
    
    Returns nil if something fails.
    
--]]
function log.open(name,aw)
  local newlog={}
  local fail=0
  -- are io commands available?
  if type(io) ~= "table" then
    -- no.
    fail=true
  else
    -- yes, io is available, form a file name.
    if name==nil then 
      -- No filename from user, find an unused name lua_???.txt
      newlog.name=log.find_new_filename("lua_","txt") 
    else  
      newlog.name=name
    end  
    newlog.path="A/CHDK/LOGS/"
    
    -- open for append unless aw is "w".
    if aw~="w" then aw="a" end
    newlog.handle,msg=io.open(newlog.path .. newlog.name, aw)
    if not newlog.handle then
      print("Error opening log: "..tostring(msg))    
      fail=true
    else
      -- New log file opened, tie printing and closing functions to it:
      
      function newlog:print(...)
        newlog.handle:write(...)
        newlog.handle:write("\n")
      end
      
      function newlog:close()
        newlog:print("------- Log closed at ",os.date("%Y/%m/%d %X")," -------")
        io.close(newlog.handle)
      end
      
      newlog.logtype="io_write"
    end
  end
  
  if fail==true then
    -- io failed for some reason, revert to print_screen
    print_screen(1)
    
    -- tie printing and closing functions to print and print_screen:
    function newlog:print(...)
        print(...)
    end
    
    function newlog:close()
      newlog:print("------- Log closed at " .. os.date("%Y/%m/%d %X") .. " -------")
      print_screen(false)
    end
    
    -- fill out the rest of the table:
    newlog.handle=nil
    newlog.logtype="print_screen"
    newlog.name="LOG_0001.TXT"
    newlog.path="A/CHDK/LOGS"
  end
  
  
  bi=get_buildinfo()
  newlog:print("\n------- Log opened at "..os.date("%Y/%m/%d %X").." -------")
  newlog:print("platform: "..bi.platform.." "..bi.platsub)
  newlog:print("version: "..bi.version.." "..bi.build_number.." built on "..bi.build_date.." "..bi.build_time)
  newlog:print("-------------------------------------------------")
  
  return newlog
end


--[[
    Find the smallest three digit zero-padded number which doesn't match any 
    existing file in A/CHDK/LOGS/ when used as a part of a file name of the 
    form headerNNN.extension. Return complete filename without path.
    
    Example:
    logfilename = log.find_new_filename("log_","txt")
    will return "log_002.txt" if log_000.txt and log_001.txt already exist.
    
    If all files up to log_999.txt are found, log_999.txt is returned.    
--]]
function log.find_new_filename(header,extension)
  local numstr  
  local filename
  
  for i=0,999 do
    -- zero pad
    if i<10 then 
      numstr = "00"..tostring(i)
    elseif i<100 then 
      numstr = "0"..tostring(i)
    else 
      numstr = tostring(i)
    end
    
    filename = tostring(header) .. numstr .. "." .. tostring(extension)
    local filepath="A/CHDK/LOGS/"..filename
    local r,msg = os.stat(filepath)
    if r==nil then 
      return filename
    end
    -- note: long for loop without forced sleep!
  end

  -- no free names found
  return filename

end
